from pyvsystems_rewards.address_factory import AddressFactory


def test_address_factory():
    # TODO this might be where we load in some real-world csv or json and
    # examine the output.
    factory = AddressFactory()
