class PoolDistribution:
    def __init__(self, pool_distribution_id, address, amount, fee, height):
        self.pool_distribution_id = pool_distribution_id
        self.address = address
        self.amount = amount
        self.fee = fee
        self.height = height
